////////////////////////////////////////////////////////////////////////////
// NoteCase notes manager project <http://notecase.sf.net>
//
// This code is licensed under BSD license.See "license.txt" for more details.
//
// File: Window class for About dialog
////////////////////////////////////////////////////////////////////////////

#include "AboutDlg.h"
#include "support.h"
#include "gui/GuiLanguage.h"
#include "ExecuteFile.h"

#ifdef _WIN32
 #include <windows.h>
#endif

extern GtkWidget *window1;

static gboolean on_website_link_clicked (GtkWidget *widget, GdkEventButton *event, gpointer user_data);
static gboolean on_mail_link_clicked (GtkWidget *widget, GdkEventButton *event, gpointer user_data);

AboutDlg::AboutDlg()
{
	Create();
}

AboutDlg::~AboutDlg()
{
	Destroy();
	gtk_window_present(GTK_WINDOW(window1)); //activate main window
}

void AboutDlg::Create()
{
	m_pDialog = create_dialog (window1);

}

GtkWidget* AboutDlg::create_dialog (GtkWidget* parent)
{
	GtkWidget *dialog;
	GtkWidget *label1;
	GtkWidget *label2;
	GtkWidget *label3;
	GtkWidget *evbox1;
	GtkWidget *label4;
	GtkWidget *evbox2;
	GtkWidget *table;

	dialog = gtk_dialog_new();
	gtk_window_set_title(GTK_WINDOW(dialog), _("About Notecase"));
	gtk_dialog_set_has_separator(GTK_DIALOG(dialog), false);
	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
	gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
#ifndef _WIN32  //TOFIX API is buggy on Win32 (kills modal dialog state)
	gtk_window_set_skip_taskbar_hint (GTK_WINDOW (dialog), TRUE);
#endif
	gtk_window_set_skip_pager_hint (GTK_WINDOW (dialog), TRUE);
	gtk_window_set_transient_for(GTK_WINDOW (dialog), GTK_WINDOW(window1));   //set parent
#if GTK_CHECK_VERSION(2,4,0) //new API TOFIX set proper version
	#ifndef _WIN32  //TOFIX API is buggy on Win32 (kills modal dialog state)
	  gtk_window_set_keep_above(GTK_WINDOW (dialog), TRUE);
	#endif
#endif

	table = gtk_table_new(2, 1, true);
	gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), table);

	//version string
	std::string strMsg(APP_NAME_STR);
	strMsg += " v.";
	strMsg += APP_VER_STR;

	label1 = gtk_label_new (strMsg.c_str());
	gtk_misc_set_alignment(GTK_MISC(label1), 0, 0.5);
	gtk_table_attach(GTK_TABLE(table), label1, 0, 1, 0, 1, (GtkAttachOptions)(GTK_EXPAND|GTK_SHRINK|GTK_FILL), (GtkAttachOptions)0, 0, 3);

	//link to the web site
	label2 = gtk_label_new ("<span foreground=\"blue\"underline=\"single\">Website: http://notecase.sourceforge.net</span>");
	gtk_label_set_use_markup(GTK_LABEL(label2), true);
	gtk_misc_set_alignment(GTK_MISC(label2), 0, 0.5);

	//put in eventbox to handle click
	evbox1 = gtk_event_box_new();
	gtk_event_box_set_above_child (GTK_EVENT_BOX(evbox1), true);
	gtk_event_box_set_visible_window(GTK_EVENT_BOX(evbox1), false);
	gtk_container_add(GTK_CONTAINER(evbox1), label2);
	gtk_table_attach(GTK_TABLE(table), evbox1, 0, 1, 1, 2, (GtkAttachOptions)(GTK_EXPAND|GTK_SHRINK|GTK_FILL), (GtkAttachOptions)0, 0, 3);

	//mail address link
	label4 = gtk_label_new ("<span foreground=\"blue\"underline=\"single\">Contact: Miroslav Rajcic</span>");
	gtk_label_set_use_markup(GTK_LABEL(label4), true);
	gtk_misc_set_alignment(GTK_MISC(label4), 0, 0.5);

	//put in eventbox to handle click
	evbox2 = gtk_event_box_new();
	gtk_event_box_set_above_child (GTK_EVENT_BOX(evbox2), true);
	gtk_event_box_set_visible_window(GTK_EVENT_BOX(evbox2), false);
	gtk_container_add(GTK_CONTAINER(evbox2), label4);
	gtk_table_attach(GTK_TABLE(table), evbox2, 0, 1, 2, 3, (GtkAttachOptions)(GTK_EXPAND|GTK_SHRINK|GTK_FILL), (GtkAttachOptions)0, 0, 3);

	//locale info
	strMsg = "Locale: ";
	strMsg += GuiLanguage::QueryLocale();

	label3 = gtk_label_new (strMsg.c_str());
	gtk_label_set_use_markup(GTK_LABEL(label3), true);
	gtk_misc_set_alignment(GTK_MISC(label3), 0.5, 0.5);
	//gtk_misc_set_padding (GTK_MISC (label3), 7, 5);
	gtk_table_attach(GTK_TABLE(table), label3, 0, 1, 3, 4, (GtkAttachOptions)(GTK_EXPAND|GTK_SHRINK|GTK_FILL), (GtkAttachOptions)0, 5, 3);


	g_signal_connect(evbox1, "button-press-event",	G_CALLBACK (on_website_link_clicked), this);
	g_signal_connect(evbox2, "button-press-event",	G_CALLBACK (on_mail_link_clicked), this);
	

	gtk_widget_show_all (dialog);
	return dialog;
}

gboolean  on_website_link_clicked (GtkWidget *widget, GdkEventButton *event, gpointer user_data)
{
#ifdef _WIN32
	ShellExecute(NULL, "open", "http://notecase.sourceforge.net", "", "", SW_SHOW);
#else
	ExecuteFile("http://notecase.sourceforge.net", "", "", NULL);
#endif

	return true;
}

gboolean on_mail_link_clicked (GtkWidget *widget, GdkEventButton *event, gpointer user_data)
{
#ifdef _WIN32
	ShellExecute(NULL, "open", "mailto:miroslav.rajcic@inet.hr", "", "", SW_SHOW);
#else
	ExecuteFile("mailto:miroslav.rajcic@inet.hr", "", "", NULL);
#endif

	return true;
}
